package repo

import (
	"encoding/json"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/adapter/repo/sqlc"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/log"
)

func convToPayments(raw json.RawMessage) ([]model.Payment, error) {
	var paymentsRetrieved []model.PaymentRetrieved
	err := json.Unmarshal(raw, &paymentsRetrieved)
	if err != nil {
		// Unexpected
		log.ErrorLog("json unmarshal error", "err", err)
		return nil, err
	}

	var payments []model.Payment
	for _, p := range paymentsRetrieved {
		payment := model.Payment{
			PayerID:        p.PayerID,
			PayerFirstName: p.PayerFirstName,
			PayerLastName:  p.PayerLastName,
			Amount:         model.MakeMoney(p.Amount, model.Currency(p.Currency)),
		}
		payments = append(payments, payment)
	}

	return payments, nil
}

func convToBenefits(raw json.RawMessage) ([]model.Benefit, error) {
	var benefitsRetrieved []model.BenefitRetrieved
	err := json.Unmarshal(raw, &benefitsRetrieved)
	if err != nil {
		// Unexpected
		log.ErrorLog("json unmarshal error", "err", err)
		return nil, err
	}

	var benefits []model.Benefit
	for _, b := range benefitsRetrieved {
		benefit := model.Benefit{
			RecipientID:        b.RecipientID,
			RecipientFirstName: b.RecipientFirstName,
			RecipientLastName:  b.RecipientLastName,
			Amount:             model.MakeMoney(b.Amount, model.Currency(b.Currency)),
		}
		benefits = append(benefits, benefit)
	}

	return benefits, nil
}

func convToExpenseRetrieved(expenseDTO *sqlc.GetExpenseByIDRow) (*model.ExpenseRetrieved, error) {
	payments, err := convToPayments(expenseDTO.Payments)
	if err != nil {
		// Unexpected
		return nil, err
	}

	benefits, err := convToBenefits(expenseDTO.Benefits)
	if err != nil {
		// Unexpected
		return nil, err
	}

	expenseRetrieved := &model.ExpenseRetrieved{
		ID:        int(expenseDTO.ID),
		CreatedAt: expenseDTO.CreatedAt,
		UpdatedAt: expenseDTO.UpdatedAt,
		Amount:    model.MakeMoney(int(expenseDTO.Amount), model.Currency(expenseDTO.Currency)),
		EventID:   int(expenseDTO.EventID),
		Detail: model.ExpenseDetail{
			Name:  expenseDTO.Name.String,
			Place: expenseDTO.Place.String,
		},
		Payments: payments,
		Benefits: benefits,
	}

	return expenseRetrieved, nil
}
